#!/bin/bash
LANG=en_US.utf8
LC_ALL=en_US.utf8
unameout="$(uname -s)"

dtt=`date --date "0 day ago" "+%Y%m%d"`
dt=`date --date "0 day ago" "+%d%b%Y"`
iwk1=`date --date "-1 day ago" "+%d%b%Y"`
iwk2=`date --date "-8 day ago" "+%d%b%Y"`
iwk3=`date --date "-15 day ago" "+%d%b%Y"`
iwk34=`date --date "-15 day ago" "+%d%b%Y"`
iwk1234=`date --date "-1 day ago" "+%d%b%Y"`

fwk1=`date --date "-7 day ago" "+%d%b%Y"`
fwk2=`date --date "-14 day ago" "+%d%b%Y"`
fwk3=`date --date "-21 day ago" "+%d%b%Y"`
fwk34=`date --date "-28 day ago" "+%d%b%Y"`
fwk1234=`date --date "-28 day ago" "+%d%b%Y"`

for wk in 1 2 34; do

if [ $wk = 1 ]; then iwk=$iwk1; fwk=$fwk1; fi
if [ $wk = 2 ]; then iwk=$iwk2; fwk=$fwk2; fi
if [ $wk = 3 ]; then iwk=$iwk3; fwk=$fwk3; fi
if [ $wk = 34 ]; then iwk=$iwk34; fwk=$fwk34; fi
if [ $wk = 1234 ]; then iwk=$iwk1234; fwk=$fwk1234; fi

iwkk=$(date -d"$iwk + 0 day" +"%d%b")
fwkk=$(date -d"$fwk + 0 day" +"%d%b")

rm cons_data/gefs_week${wk}_elr.nc
rm cons_data/gefs_week${wk}_elr_groc.nc

cat>gen_elr_masked.py<<eofPY
#!/usr/bin/env python
# coding: utf-8

import xcast as xc 
import xarray as xr 
import cartopy.crs as ccrs 
import numpy as np
import matplotlib.pyplot as plt
#import rioxarray
#import geopandas as gpd
from shapely.geometry import mapping

wlon= $1
elon= $2
slat = $3
nlat = $4

longrid = np.arange(wlon, elon + 10,10)
latgrid = np.arange(slat, nlat + 10,10)

longridl = longrid.tolist()
latgridl = latgrid.tolist()

wl = wlon
el = elon
sl = slat
nl = nlat


model1 = xr.open_dataset('train_data/gefs_week${wk}_hind.nc')
obs1 = xr.open_dataset('train_data/cchirps_week${wk}_hind.nc')
fmodel1 = xr.open_dataset('train_data/gefs_week${wk}_fcst.nc')

model = model1.precip.expand_dims({'M':[0]})
fmodel = fmodel1.precip.expand_dims({'M':[0]})
obs = obs1.precip.expand_dims({'M':[0]})

drymask = xc.drymask(obs, dry_threshold=0.1, quantile_threshold=0.3)
obs = obs * drymask

drymask = xc.drymask(model, dry_threshold=0.1, quantile_threshold=0.3)
model = model * drymask

model = model.sel(lon=slice(wl, el), lat=slice(sl, nl))
fmodel = fmodel.sel(lon=slice(wl, el), lat=slice(sl, nl))
obs = obs.sel(lon=slice(wlon, elon), lat=slice(slat, nlat))

mask_missing = model.mean('time', skipna=False).mean('M', skipna=False)
mask_missing = xr.ones_like(mask_missing).where(~np.isnan(mask_missing), other=np.nan )
model = model * mask_missing

model = xc.regrid(model, obs.lon, obs.lat)
fmodel = xc.regrid(fmodel, obs.lon, obs.lat)

hindcasts_prob = []
i=1
for xtrain, ytrain, xtest, ytest in xc.CrossValidator(model, obs, window=5):
    print("window {}".format(i))
    i += 1
    reg = xc.ELR()
    reg.fit(xtrain, ytrain)
    probs =  reg.predict_proba(xtest)
    hindcasts_prob.append(probs.isel(time=2))
hindcasts_prob = xr.concat(hindcasts_prob, 'time')

hindcasts_prob = xc.gaussian_smooth(hindcasts_prob, kernel=3)
obs = xc.gaussian_smooth(obs, kernel=3)

ohc = xc.OneHotEncoder() 
ohc.fit(obs)
T = ohc.transform(obs)
clim = xr.ones_like(T) * 0.333

fprobs =  reg.predict_proba(fmodel)

fprobs20 =  reg.predict_proba(fmodel,quantile=0.2)
fprobs80 =  1-(reg.predict_proba(fmodel,quantile=0.8))
fprobs10 =  reg.predict_proba(fmodel,quantile=0.1)
fprobs90 =  1-(reg.predict_proba(fmodel,quantile=0.9))

import matplotlib.ticker as mticker
from cartopy.mpl.gridliner import LONGITUDE_FORMATTER, LATITUDE_FORMATTER

import sys
sys.path.append('./libs')

from pupper import puview
wet = ['80', '90']
for p in wet:
    fr = eval("fprobs"+p).transpose("lat", "lon", "time", "M")
    frr = fr.rename("Exceedance_Probability")
    frrr = frr.mean(dim=['time', 'M'])*100
    # view probabilistic and view, with cross-dateline=True will force back to (0,360) for plotting , won't change the original data tho
    ax = puview(frrr, x_feature_dim=None, coastlines=True, ocean=False, title = f"GEFS, Week-${wk}, ELR > {p}th, Valid: ${iwk} - ${fwk}")
    gl = ax.gridlines(crs=ccrs.PlateCarree(), linewidth=1, color='blue', alpha=0.5, linestyle='--', draw_labels=True)
    gl.top_labels = False
    gl.left_labels = False
    gl.right_labels=True
    gl.xlines = True
    gl.xlocator = mticker.FixedLocator(longridl)
    gl.ylocator = mticker.FixedLocator(latgridl)
    gl.xformatter = LONGITUDE_FORMATTER
    gl.yformatter = LATITUDE_FORMATTER
    gl.xlabel_style = {'color': 'black'}
    gl.top_labels = False
    gl.left_labels = True
    gl.right_labels=False
    gl.bottom_labels=True
    plt.savefig(f'fig_dir/gefs_week_${wk}_elr_{p}.png', dpi=300)

from plower import puview
dry = ['10', '20']
for p in dry:
    fr = eval("fprobs"+p).transpose("lat", "lon", "time", "M")
    frr = fr.rename("Exceedance_Probability")
    frrr = frr.mean(dim=['time', 'M'])*100
    # view probabilistic and view, with cross-dateline=True will force back to (0,360) for plotting , won't change the original data tho
    ax = puview(frrr, x_feature_dim=None, coastlines=True, ocean=False, title = f"GEFS, Week-${wk}, ELR < {p}th, Valid: ${iwk} - ${fwk}")
    gl = ax.gridlines(crs=ccrs.PlateCarree(), linewidth=1, color='blue', alpha=0.5, linestyle='--', draw_labels=True)
    gl.top_labels = False
    gl.left_labels = False
    gl.right_labels=True
    gl.xlines = True
    gl.xlocator = mticker.FixedLocator(longridl)
    gl.ylocator = mticker.FixedLocator(latgridl)
    gl.xformatter = LONGITUDE_FORMATTER
    gl.yformatter = LATITUDE_FORMATTER
    gl.xlabel_style = {'color': 'black'}
    gl.top_labels = False
    gl.left_labels = True
    gl.right_labels=False
    gl.bottom_labels=True
    plt.savefig(f'fig_dir/gefs_week_${wk}_elr_{p}.png', dpi=300)

from prob import pview_probabilistic
ax = pview_probabilistic(fprobs.isel(time=0), ocean=False, title='GEFS, Week-${wk}, ELR, Valid: ${iwk} - ${fwk}')
gl = ax.gridlines(crs=ccrs.PlateCarree(), linewidth=1, color='blue', alpha=0.5, linestyle='--', draw_labels=True)
gl.top_labels = False
gl.left_labels = False
gl.right_labels=True
gl.xlines = True
gl.xlocator = mticker.FixedLocator(longridl)
gl.ylocator = mticker.FixedLocator(latgridl)
gl.xformatter = LONGITUDE_FORMATTER
gl.yformatter = LATITUDE_FORMATTER
gl.xlabel_style = {'color': 'black'}
gl.top_labels = False
gl.left_labels = True
gl.right_labels=False
gl.bottom_labels=True
plt.savefig('fig_dir/gefs_week_${wk}_elr.png', dpi=300)

from prpssview import rview
clim_rps = xc.RankProbabilityScore(clim, T)
pred_rps = xc.RankProbabilityScore(hindcasts_prob, T)
rpss = 1 - pred_rps / clim_rps
ax = rview(rpss, ocean=False, title='GEFS, Week-${wk}, ELR-RPSS, Valid: ${iwkk} - ${fwkk}')
gl = ax.gridlines(crs=ccrs.PlateCarree(), linewidth=2, color='black', alpha=0.5, linestyle='--', draw_labels=True)
gl.xlocator = mticker.FixedLocator(longridl)
gl.ylocator = mticker.FixedLocator(latgridl)
gl.xformatter = LONGITUDE_FORMATTER
gl.yformatter = LATITUDE_FORMATTER
gl.xlabel_style = {'color': 'black'} 
gl.top_labels = False
gl.left_labels = True
gl.right_labels=False
gl.bottom_labels=True
gl.xlines = True
plt.savefig('fig_dir/gefs_week_${wk}_elr_rpss.png')

from pgrocsview import rview
groc = xc.GROCS(hindcasts_prob, T)
ax = rview(groc, ocean=False, title='GEFS, Week-${wk}, ELR-GROCS, Valid: ${iwkk} - ${fwkk}')
gl = ax.gridlines(crs=ccrs.PlateCarree(), linewidth=2, color='black', alpha=0.5, linestyle='--', draw_labels=True)
gl.xlocator = mticker.FixedLocator(longridl)
gl.ylocator = mticker.FixedLocator(latgridl)
gl.xformatter = LONGITUDE_FORMATTER
gl.yformatter = LATITUDE_FORMATTER
gl.xlabel_style = {'color': 'black'}
gl.top_labels = False
gl.left_labels = True
gl.right_labels=False
gl.bottom_labels=True
gl.xlines = True
plt.savefig('fig_dir/gefs_week_${wk}_elr_groc.png')


from rocc import view_mroc
ds = hindcasts_prob.assign_coords(M=(['BN', 'NN', 'AN']))
view_mroc(ds, T, savefig='fig_dir/gefs_week_${wk}_elr_roc.png')

xc.view_reliability(hindcasts_prob, T)
plt.savefig('fig_dir/gefs_week_${wk}_elr_relib.png', dpi=100)

import os
fr = fprobs.assign_coords(M=([0, 1, 2]))
frr = fr.transpose("lat", "lon", "time", "M")
ttt = frr.rename("prob")
ttt.to_netcdf('cons_data/gefs_week${wk}_elr.nc')

ttt = groc.rename("skill")
ttt.to_netcdf('cons_data/gefs_week${wk}_elr_groc.nc')
eofPY

python gen_elr_masked.py

convert -trim fig_dir/gefs_week_${wk}_elr.png fig_dir/gefs_week_${wk}_elr.png
convert -trim fig_dir/gefs_week_${wk}_elr_rpss.png fig_dir/gefs_week_${wk}_elr_rpss.png
convert -trim fig_dir/gefs_week_${wk}_elr_groc.png fig_dir/gefs_week_${wk}_elr_groc.png
convert -trim fig_dir/gefs_week_${wk}_elr_relib.png fig_dir/gefs_week_${wk}_elr_relib.png

convert -bordercolor white -border 10 fig_dir/gefs_week_${wk}_elr.png fig_dir/gefs_week_${wk}_elr.png
convert -bordercolor white -border 10 fig_dir/gefs_week_${wk}_elr_rpss.png fig_dir/gefs_week_${wk}_elr_rpss.png
convert -bordercolor white -border 10 fig_dir/gefs_week_${wk}_elr_groc.png fig_dir/gefs_week_${wk}_elr_groc.png
convert -bordercolor white -border 10 fig_dir/gefs_week_${wk}_elr_relib.png fig_dir/gefs_week_${wk}_elr_relib.png

for th in 10 20 80 90; do
convert -trim fig_dir/gefs_week_${wk}_elr_${th}.png fig_dir/gefs_week_${wk}_elr_${th}.png
convert -bordercolor white -border 10 fig_dir/gefs_week_${wk}_elr_${th}.png fig_dir/gefs_week_${wk}_elr_${th}.png
done


done
